'use strict';

/**
 * @file serviceWorker register no-cache solution
 * @author mj(zoumiaojiang@gmail.com)
 */

var etpl = require('etpl');
var fs = require('fs');
var path = require('path');
var babel = require('babel-core');

var cwd = process.cwd();

/**
 * 对于小于 100 的数字向左补全0
 *
 * @param  {number} value 数字
 * @return {string}       补全后的字符串
 */
function padding(value) {
    return value < 10 ? `0${value}` : value;
}

/**
 * 获取时间戳版本号
 *
 * @return {string} 版本号
 */
function getVersion() {
    var d = new Date();

    return '' + d.getFullYear() + padding(d.getMonth() + 1) + padding(d.getDate()) + padding(d.getHours()) + padding(d.getMinutes()) + padding(d.getSeconds());
}

/**
 * babel 编译
 *
 * @param {string} source 源代码
 * @return {string} 编译后的代码
 */
function babelCompiler(source) {
    return babel.transform(source, {
        comments: false,
        minified: true,
        presets: [['env', {
            targets: {
                node: 3
            },
            modules: false
        }]]
    }).code;
}

/**
 * sw Register 插件
 *
 * @constructor
 * @param {Object} options 参数
 */
function SwRegisterPlugin() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var filePath = path.resolve(cwd, options.filePath || './src/sw-register.js');

    if (!fs.existsSync(filePath)) {
        filePath = path.resolve(__dirname, 'templates', 'sw-register.js');
    }
    this.filePath = filePath;
    this.fileName = path.basename(filePath);
    this.version = options.version || getVersion();
    this.prefix = options.prefix;
}

SwRegisterPlugin.prototype.apply = function (compiler) {
    var me = this;
    var swRegisterEntryFilePath = path.resolve(__dirname, 'templates', 'sw-register-entry.js.tpl');
    var swRegisterFilePath = me.filePath;

    compiler.plugin('emit', function (compilation, callback) {
        var prefix = me.prefix || compilation.outputOptions.publicPath || '';
        var publicPath = me.publicPath = (prefix + '/').replace(/\/{1,}/g, '/');
        var con = fs.readFileSync(swRegisterFilePath, 'utf-8');
        var version = me.version;

        /* eslint-disable max-nested-callbacks */
        con = babelCompiler(con).replace(/(['"])([^\s\;\,\(\)]+?\.js)\1/g, function (item) {
            var swJs = RegExp.$2;

            if (swJs[0] !== '/') {
                throw new Error('Js path in `sw-register.js` must be a absolute path');
            }

            if (swJs.indexOf(publicPath) < 0) {
                var ret = item.replace(swJs, (publicPath + '/' + swJs).replace(/\/{1,}/g, '/').replace(/\.js/g, function (ext) {
                    return `${ext}?v=${version}`;
                }));

                return ret;
            }

            return item.replace(/\.js/g, function (ext) {
                return `${ext}?v=${version}`;
            });
        });

        compilation.assets[me.fileName] = {
            source() {
                return con;
            },
            size() {
                return con.length;
            }
        };

        Object.keys(compilation.assets).forEach(function (asset) {

            if (asset.indexOf('.html') > -1) {
                var htmlContent = compilation.assets[asset].source();
                var swRegisterEntryFileTpl = fs.readFileSync(swRegisterEntryFilePath, 'utf-8');
                var swRegisterEntryFileContent = etpl.compile(swRegisterEntryFileTpl)(me);

                htmlContent = htmlContent.replace(/<\/body>/, `${swRegisterEntryFileContent}</body>`);

                compilation.assets[asset] = {
                    source() {
                        return htmlContent;
                    },
                    size() {
                        return htmlContent.length;
                    }
                };
            }
        });
        callback();
    });
};

module.exports = SwRegisterPlugin;